# RefreshLess implementation using the [Hotwire Turbo library](https://turbo.hotwired.dev/)

## Requirements

* [Drupal core](https://www.drupal.org/project/drupal) 10.1 or newer

* [PHP](https://www.php.net/) 8.1 or newer

* [Composer](https://getcomposer.org/)

* A front-end package manager that supports both [the Plug'n'Play install strategy](https://yarnpkg.com/features/pnp) and [workspaces](https://yarnpkg.com/features/workspaces):

  * [Yarn 3 or newer](https://yarnpkg.com/getting-started/install) (Yarn 1 *will not work*; Yarn 2 *might work* but has not been tested)

  * [pnpm](https://pnpm.io/) may also work but has not been tested

We generally recommend [DDEV](https://ddev.com/) because it provides most or all of the above for you out of the box, and [helps you get a Drupal project up and running](https://www.drupal.org/docs/develop/local-server-setup/docker-based-development-environments-for-macos-linux-and-windows) much faster while also isolating it from the rest of your operating system via containerization.

## Installation

The first step is to require this via Composer as you would any other Drupal
module.

*The next step should be performed before telling Drupal to install this module;
this pulls in the Turbo library via a package manager. The instructions are for
[Yarn](https://yarnpkg.com/), but can be adapted for [pnpm](https://pnpm.io/)
or other package managers.*

The following assumes you're using [Yarn
workspaces](https://yarnpkg.com/features/workspaces) from the root of your
Drupal installation, i.e. where your root `composer.json` is located, one level
above the web directory. You must also be using [the Plug'n'Play install
strategy](https://yarnpkg.com/features/pnp), which is the default in modern
Yarn. You'll need to have a `package.json` in the root directory containing a
`workspaces` array:

```json
{
  "workspaces": [
    "web/modules/contrib/refreshless/**!(vendor)/*"
  ]
}
```

If you already have other workspace paths defined, just merge that in.

If you need fixes for [known issues](#known-issues) that require patching Turbo,
please see the [Patching section](#patching) before continuing.

Then, run `yarn install` and Turbo will be downloaded and extracted to the
correct location. This is accomplished via the
[`@consensus.enterprises/pnp-vendorize`](https://www.npmjs.com/package/@consensus.enterprises/pnp-vendorize)
package.

Once that's done, you can install this module in Drupal and it should take over
handling page requests.

## Known issues

* [CSS files merged into the `<head>` always appended instead of preserving full page load order, causing CSS specificity issues](https://www.drupal.org/project/refreshless/issues/3399314); workaround is to enable CSS aggregation because Drupal will then order the CSS files correctly within their aggregate groups.

* [In-page anchors and `hashchange` events can cause full page visits or otherwise behave unexpectedly](https://www.drupal.org/project/refreshless/issues/3416085); workaround is to apply `patches/@hotwired/turbo/turbo-8-pull-1125-anchor-hashchange.patch` or our combined patch; see [Patching](#patching) section.

* [Visiting pages with 4xx response codes causes re-evaluation of `<head>` JavaScript](https://www.drupal.org/project/refreshless/issues/3422964); workaround is to apply `patches/@hotwired/turbo/turbo-8-issue-1190-render-error-script-eval.patch` or our combined patch; see [Patching](#patching) section.

* JavaScript aggregation has multiple issues and is not yet compatible:

  * We force all JavaScript into the `<head>` but add the `defer` attribute as [recommended by the Turbo documentation](https://turbo.hotwired.dev/handbook/building#loading-your-application%E2%80%99s-javascript-bundle); this causes Drupal core to throw its hands up in the air and not aggregate any of the JavaScript since it doesn't know what to do with the attribute - see [this core issue #1587536 for a patch](https://www.drupal.org/project/drupal/issues/1587536), but...

  * Once core is patched to allow aggregating JavaScript with the `defer` attribute, [enabling aggregation results in multiple evaluations of the same libraries over time as you navigate](https://www.drupal.org/project/refreshless/issues/3414538).

## Patching

The following describes how to apply our Turbo patches [via
Yarn](https://yarnpkg.com/features/patching) but can also be [adapted for
pnpm](https://pnpm.io/cli/patch). We want to eventually contribute to Turbo and
remove the need for patches, but until that time, patching will be required if
you need fixes for [known issues](#known-issues). Note that Yarn doesn't allow
including a `"resolutions"` field in anything but the top-level `package.json`,
so you have to merge this yourself into yours:

```json
"resolutions": {
  "@hotwired/turbo@^8.0.3": "patch:@hotwired/turbo@npm%3A8.0.3#./web/modules/contrib/refreshless/modules/refreshless_turbo/patches/@hotwired/turbo/turbo-8-refreshless-combined.patch"
}
```

Then run `yarn install` and Yarn will patch Turbo for you.

Note that this assumes RefreshLess is installed to `web/modules/contrib` so
update the path if needed. Also note that the `"resolutions"` field only
supports exact versions and not version ranges, so when Turbo is upgraded to a
new version - including a minor version - you'll have to update both instances
of the hard-coded version above, in both the key and the value.
